;==================================
; Nibbles by Cullen Sauls (For The TI-82)
; Webpage: http://cullen.macross2000.org
; E-Mail: sauls@tamu.edu
; ICQ: 11130342 (rbit [YFL])
; AIM: OrbitYFL
;
; Based on Oskar Liljeblad's Nibbles 85, but no code was copied.
;==================================
#include "ti82.h"                    ; include ASH aliases
#include "keys.inc"                  ; include key aliases

.org    START_ADDR           ; (cr)ash programs start at START_ADDR
.db     "Nibbles v1.1",0     ; title

;--- Aliases ---;
Right = 3
Left = 2
Up = 4
Down = 1

;-----------;
; Variables ;
;-----------;

Level = TEXT_MEM
Speed = TEXT_MEM+1
string = TEXT_MEM+2
NumGoodies = TEXT_MEM+8
SnakeDir = TEXT_MEM+9
SnakeLength = TEXT_MEM+10
Stall = TEXT_MEM+11
GoodieX = TEXT_MEM+12
GoodieY = TEXT_MEM+13
Score = TEXT_MEM+14         ;Word
SnakeCoords = TEXT_MEM+16

Start:
 ld a,$8C
 out (2),a
;
 call ClearGraphMem
 LD DE,ZCP_Data
 LD HL,GRAPH_MEM
 Call DecodeZCP
 ROM_CALL(DISP_GRAPH)

 ROM_CALL(KEY_HAND)

MenuScreen:
 
 ROM_CALL(CLEARLCD)

 LD HL,$0000
 LD (CURSOR_X),HL
 LD HL,MenuStrings
 ROM_CALL(D_ZM_STR)
 LD de,$0600
 LD (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 LD de,$170D
 LD (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 LD de,$1D00
 LD (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 LD de,$2313
 LD (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 LD de,$2913
 LD (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 LD de,$3900
 LD (CURSOR_X),de
 ROM_CALL(D_ZM_STR)

 ld de,$1700
 ld (CURSOR_X),de
 ld a,$CF
 ROM_CALL(M_CHARPUT)
 ld a,$2F
 ROM_CALL(M_CHARPUT)
 ld a,$05
 ROM_CALL(M_CHARPUT)

 LD a,1
 LD (Level),a
 LD a,3
 LD (Speed),a

;<---------- Line 102 ---------->

;-------------;
; Menu Screen ;
;-------------;
MenuNow:
 ld a,1
 call LevelChange
 ld a,G_ZOOM
 call SetSpeed
GetInfoLoop:
 call GET_KEY
 CP G_LEFT
 call z,LevSub
 CP G_RIGHT
 call z,LevAdd
 CP G_YEDIT
 call z,SetSpeed
 CP G_WINDOW
 call z,SetSpeed
 CP G_ZOOM
 call z,SetSpeed
 CP G_TRACE
 call z,SetSpeed
 CP G_GRAPH
 call z,SetSpeed
 CP G_CLEAR
 jp z,Exit
 CP G_ENTER
 jp z,StartGame
 jr GetInfoLoop
StartGame:
 ld a,3
 ld (SnakeDir),a
 ld a,4
 ld (Stall),a
 ld (SnakeCoords),a
 ld a,6
 ld (SnakeCoords+1),a
 ld a,5
 ld (SnakeLength),a
 xor a
 ld (NumGoodies),a
 call DrawLevel
 call PlaceGoodie
MainGameLoop:
 call UpdateSnake
 call GET_KEY
 cp G_CLEAR
 jp z,Exit
 cp G_LEFT
 call z,ChangeDir
 cp G_RIGHT
 call z,ChangeDir
 cp G_UP
 call z,ChangeDir
 cp G_DOWN
 call z,ChangeDir
 call SpeedOfGame
 jr MainGameLoop
;-------------;
; Subroutines ;
;-------------;
LevAdd:
 LD A,(Level)
 inc a
 cp 11
 jr nz,$+4
 ld a,1
 LD (Level),A
 xor a
 jr LevelChange
LevSub:
 ld a,(Level)
 dec a
 jr nz,$+4
 ld a,10
 LD (Level),A
 xor a
LevelChange:
 ld hl,$1754
 ld (CURSOR_X),hl
 ld hl,Space
 ROM_CALL(D_ZM_STR)
 LD HL,$1750
 LD (CURSOR_X),HL
 LD A,(Level)
 ld l,a
 ld h,0
 call DM_HL_DECI
 ret
SetSpeed:
 ld b,a
 ld a,$36
 sub b
 ld b,a
 ld a,6
 sub b
 ld (Speed),a
 ld b,a
 ld hl,SpeedStrings-18
 ld de,18
DispSpeedLoop:
 add hl,de
 djnz DispSpeedLoop
 ld de,$1D44
 ld (CURSOR_X),de
 ROM_CALL(D_ZM_STR)
 ret
DrawLevel:
 ROM_CALL(CLEARLCD)
 call ClearGraphMem
 ld hl,$88B8
 ld a,$ff
 ld b,24
DLLoop1:
 ld (hl),a
 inc hl
 djnz DLLoop1
 ld b,60
 ld de,11
DLLoop2:
 ld a,%11000000
 ld (hl),a
 add hl,de
 ld a,%11
 ld (hl),a
 inc hl
 djnz DLLoop2
 ld a,$ff
 ld b,24
DLLoop3:
 ld (hl),a
 inc hl
 djnz DLLoop3
 ld a,(Level)
 dec a
 dec a
 call z,DrawLevel2
 dec a
 call z,DrawLevel3
 dec a
 call z,DrawLevel4
 dec a
 call z,DrawLevel5
 dec a
 call z,DrawLevel6
 dec a
 call z,DrawLevel7
 dec a
 call z,DrawLevel8
 dec a
 call z,DrawLevel9
 dec a
 call z,DrawLevel10
 ROM_CALL(DISP_GRAPH)
 ret
DrawLevel2:
 ld hl,$8A22
 ld de,4
 ld c,2
DL2Loop1:
 ld a,$ff
 ld b,8
DL2Loop2:
 ld (hl),a
 inc hl
 djnz DL2Loop2
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL2Loop1
 ret
DrawLevel3:
 ld hl,$894A
 ld b,40
 ld a,%110000
DL3Loop:
 ld (hl),a
 ld de,7
 add hl,de
 ld (hl),a
 ld de,5
 add hl,de
 djnz DL3Loop
 ret
DrawLevel4:
 ld hl,$88D2
 ld de,12
 ld b,30
 ld a,%1100
DL4Loop1:
 ld (hl),a
 add hl,de
 djnz DL4Loop1
 ld hl,$894D
 ld de,5
 ld c,2
DL4Loop2:
 ld a,$ff
 ld b,7
DL4Loop3:
 ld (hl),a
 inc hl
 djnz DL4Loop3
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL4Loop2
 ld hl,$8B10
 ld de,5
 ld c,2
DL4Loop4:
 ld a,$ff
 ld b,7
DL4Loop5:
 ld (hl),a
 inc hl
 djnz DL4Loop5
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL4Loop4
 ld hl,$8B9D
 ld de,-12
 ld b,30
 ld a,%110000
DL4Loop6:
 ld (hl),a
 add hl,de
 djnz DL4Loop6
 ret
DrawLevel5:
 ld hl,$897B
 ld de,7
 ld c,2
DL5Loop1:
 ld a,$ff
 ld b,5
DL5Loop2:
 ld (hl),a
 inc hl
 djnz DL5Loop2
 ld a,%11000000
 ld (hl),a
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL5Loop1
 ld hl,$89AA
 ld de,6
 ld a,%1100
 ld b,48
DL5Loop3:
 ld (hl),a
 add hl,de
 djnz DL5Loop3
 ld hl,$8AE3
 ld de,7
 ld c,2
DL5Loop4:
 ld a,$ff
 ld b,5
DL5Loop5:
 ld (hl),a
 inc hl
 djnz DL5Loop5
 ld a,%11000000
 ld (hl),a
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL5Loop4
 ret
DrawLevel6:
 ld hl,$88B8+25
 ld c,22
DL6Loop1:
 ld a,%11
 ld b,5
DL6Loop2:
 ld (hl),a
 inc hl
 inc hl
 djnz DL6Loop2
 dec c
 inc hl
 inc hl
 xor a
 or c
 jr nz,DL6Loop1
 ld hl,$88B8+481
 ld c,22
DL6Loop3:
 ld a,%11
 ld b,5
DL6Loop4:
 ld (hl),a
 inc hl
 inc hl
 djnz DL6Loop4
 dec c
 inc hl
 inc hl
 xor a
 or c
 jr nz,DL6Loop3
 ld a,4
 ld (SnakeDir),a
 xor a
 ret
DrawLevel7:
 ld hl,$88D6
 ld de,12
 ld a,%110000
 ld b,15
DL7Loop:
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 add hl,de
 add hl,de
 djnz DL7Loop
 ret
DrawLevel8:
 ld hl,$88B8+25
 ld a,%11
 ld de,4
 ld b,45
DL8Loop1:
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 djnz DL8Loop1
 ld hl,$88B8+207
 ld a,%11
 ld de,4
 ld b,45
DL8Loop2:
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 add hl,de
 djnz DL8Loop2
 ret
DrawLevel9:
 ld b,10
 ld c,50
DL9Loop1:
 call DrawBlock
 inc b
 inc b
 dec c
 dec c
 ld a,c
 cp 12
 jr nz,DL9Loop1
 ld b,48
 ld c,50
DL9Loop2:
 call DrawBlock
 inc b
 inc b
 dec c
 dec c
 ld a,c
 cp 12
 jr nz,DL9Loop2
 ret
DrawLevel10:
 ld hl,$88B8+25
 ld c,15
DL10Loop1:
 ld de,4
 ld a,%11
 ld b,2
DL10Loop2:
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 djnz DL10Loop2
 ld de,24
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL10Loop1
 ld hl,$88B8+51
 ld c,15
DL10Loop3:
 ld a,%11
 ld de,4
 ld b,2
DL10Loop4:
 ld (hl),a
 add hl,de
 ld (hl),a
 add hl,de
 add hl,de
 djnz DL10Loop4
 ld de,24
 add hl,de
 dec c
 xor a
 or c
 jr nz,DL10Loop3
 ld a,4
 ld (SnakeDir),a
 xor a
 ret
;
UpdateSnake:
 ld hl,SnakeCoords+106
 ld de,SnakeCoords+108
 ld bc,107
 lddr
 ld a,(SnakeDir)
 cp Up
 call z,MoveUp
 cp Down
 call z,MoveDown
 cp Left
 call z,MoveLeft
 cp Right
 call z,MoveRight
 ld hl,SnakeCoords
 ld b,(hl)
 inc hl
 ld c,(hl)
 call DrawBlock
 ld a,(Stall)
 or a
 jr z,EraseTail
 dec a
 ld (Stall),a
EndEraseTail:
 ROM_CALL(DISP_GRAPH)
 ret
EraseTail:
 call GetEndCoords
 ld c,(hl)
 dec hl
 ld b,(hl)
 call EraseBlock
 jr EndEraseTail
MoveUp:
 ld hl,SnakeCoords+1
 inc (hl)
 inc (hl)
 ld c,(hl)
 dec hl
 ld b,(hl)
 call PointTest
 or a
 ret z
 call CheckForGoodie
 pop hl
 pop hl
 jp GameOver
MoveDown:
 ld hl,SnakeCoords+1
 dec (hl)
 dec (hl)
 ld c,(hl)
 dec hl
 ld b,(hl)
 call PointTest
 or a
 ret z
 call CheckForGoodie
 pop hl
 pop hl
 jp GameOver
MoveLeft:
 ld hl,SnakeCoords
 dec (hl)
 dec (hl)
 ld b,(hl)
 inc hl
 ld c,(hl)
 call PointTest
 or a
 ret z
 call CheckForGoodie
 pop hl
 pop hl
 jp GameOver
MoveRight:
 ld hl,SnakeCoords
 inc (hl)
 inc (hl)
 ld b,(hl)
 inc hl
 ld c,(hl)
 call PointTest
 or a
 ret z
 call CheckForGoodie
 pop hl
 pop hl
 jp GameOver
GetEndCoords:
 ld a,(SnakeLength)
 add a,a
 ld hl,SnakeCoords
 ld e,a
 ld d,0
 add hl,de
 dec hl
 ret
;
CheckForGoodie:
 ld a,(SnakeCoords)
 ld e,a
 ld a,(SnakeCoords+1)
 ld d,a
 ld a,(GoodieX)
 ld l,a
 ld a,(GoodieY)
 ld h,a
 call CP_HL_DE
 ret nz
 ld hl,(Score)
 inc hl
 ld (Score),hl
 ld a,(NumGoodies)
 inc a
 ld (NumGoodies),a
 cp 10
 jr z,AdvanceLevel
 call PlaceGoodie
 ld a,(SnakeLength)
 add a,5
 ld (SnakeLength),a
 pop hl
 ret
;
AdvanceLevel:
 ld de,$0101
 ld (CURSOR_ROW),de
 ld hl,AdvanceText
 ROM_CALL(D_ZT_STR)
 ld de,$0103
 ld (CURSOR_ROW),de
 ROM_CALL(D_ZT_STR)
 ld de,$0105
 ld (CURSOR_ROW),de
 ROM_CALL(D_ZT_STR)
 ld de,$0903
 ld (CURSOR_ROW),de
 ld hl,(Score)
 ROM_CALL(D_HL_DECI)
 call WaitForEnter
 ld a,(Level)
 inc a
 cp 11
 jr nz,$+4
 ld a,1
 ld (Level),a
 pop hl
 pop hl
 pop hl
 jp StartGame
;
PlaceGoodie:
 ld a,r
 srl a
 and 47
 add a,a
 ld b,a
 ld a,r
 srl a
 and 31
 add a,a
 ld c,a
 push bc
 call PointTest
 pop bc
 or a
 jr nz,PlaceGoodie
 ld a,b
 ld (GoodieX),a
 ld a,c
 ld (GoodieY),a
 call DrawBlock
 ROM_CALL(DISP_GRAPH)
 ret
;
ChangeDir:
 ld (SnakeDir),a
 ret
;
SpeedOfGame:
 ld a,(Speed)
 dec a
 ret z
 add a,a
 ld b,a
SOGLoop:
 halt
 djnz SOGLoop
 ret
;
GameOver:
 ld de,$0101
 ld (CURSOR_ROW),de
 ld hl,GameOverText
 ROM_CALL(D_ZT_STR)
 ld de,$0103
 ld (CURSOR_ROW),de
 ROM_CALL(D_ZT_STR)
 ld de,$0105
 ld (CURSOR_ROW),de
 ROM_CALL(D_ZT_STR)
 ld de,$0903
 ld (CURSOR_ROW),de
 ld hl,(Score)
 ROM_CALL(D_HL_DECI)
 call WaitForEnter
 jp MenuScreen
;
WaitForEnter:
 call GET_KEY
 cp G_ENTER
 jr nz,WaitForEnter
 ret
;
DM_HL_DECI:
 push de
 push hl
 ld de,string+5         ;start at the end
 xor a                  ;ld a, 0
 ld (de),a              ;zero terminated string
DMRepeat:
 call UNPACK_HL         ;unpack one digit of hl to a
 add a,'0'              ;add it with ASCII char 0
 dec de                 ;previous byte
 ld (de),a              ;load the char
 ld a,h                 ;is hl 0?
 or l                   ;|
 jr nz,DMRepeat         ;loop
 ex de,hl               ;exchange so hl points to string
 ROM_CALL(D_ZM_STR)     ;display string
 pop hl                 ;recall hl
 pop de                 ;recall de
 ret                    ;return to caller
;
Exit:
 ROM_CALL(CLEARLCD)
 call ClearGraphMem
 ld de,0
 ld (CURSOR_ROW),de
 ld hl,EndText
 set 3,(IY+05)
 ROM_CALL(D_ZT_STR)
 res 3,(IY+05)
 ROM_CALL(KEY_HAND)
 ret
;
ClearGraphMem:
 ld HL,GRAPH_MEM
 xor A
 ld (HL),A
 ld DE,GRAPH_MEM + 1
 ld BC,$2FF
 ldir                          ; clear GRAPH_MEM
 ret
DrawBlock:
 push bc
 call PointOn
 inc b
 call PointOn
 dec b
 inc c
 call PointOn
 inc b
 call PointOn
 pop bc
 ret
EraseBlock:
 push bc
 call PointOff
 inc b
 call PointOff
 dec b
 inc c
 call PointOff
 inc b
 call PointOff
 pop bc
 ret
PointOn:
 push bc
 call FIND_PIXEL
 ld de,GRAPH_MEM
 add hl,de
 or (hl)
 ld (hl),a
 pop bc
 ret
PointOff:
 push bc
 call FIND_PIXEL
 ld de,GRAPH_MEM
 add hl,de
 cpl
 and (hl)
 ld (hl),a
 pop bc
 ret
PointTest:
 call FIND_PIXEL
 ld de,GRAPH_MEM
 add hl,de
 and (hl)
 ret
;
;---------------------------------;
; All text and picture data here! ;
;---------------------------------;
;
ZCP_Data:
#include "Nibpic.asm"
#include "ZCPdecod.asm"
;
MenuStrings:
 .DB "Welcome To Nibbles  v1.1",0
 .DB "By: Cullen Sauls",0
 .DB "Arrows:  Level",0
 .DB "Top Buttons:  Speed",0
 .DB "Enter:  Start",0
 .DB "Clear:  Quit",0
 .DB "<freeweb.pdq.net/sauls1>",0
SpeedStrings:
 .DB "Fastest          ",0
 .DB "Fast             ",0
 .DB "Normal           ",0
 .DB "Slow             ",0
 .DB "Slowest          ",0
Space:
 .DB "   ",0
PauseText:
 .db "Paused",0
AdvanceText:
 .db "Level cleared!",0
 .db "Score:",0
 .db "Press Enter...",0
GameOverText:
 .db "Game Over",0
 .db "Score:",0
 .db "Press Enter...",0
EndText:
 .db "   Nibbles By   "
 .db "  Cullen Sauls  "
 .db "                "
 .db " Special Thanks "
 .db "Oskar  Liljeblad"
 .db "      for       "
 .db "   Nibbles 85   ",0
;
#include "graph.h"
;
.end
